package com.xxfc.platform.order.service;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson.JSON;
import com.github.wxiaoqi.security.admin.feign.dto.AppUserDTO;
import com.github.wxiaoqi.security.admin.feign.rest.UserRestInterface;
import com.github.wxiaoqi.security.auth.client.config.UserAuthConfig;
import com.xxfc.platform.activity.entity.ActivityInfo;
import com.xxfc.platform.activity.feign.ActivityFeign;
import com.xxfc.platform.activity.vo.ActivityInfoRuleVo;
import com.xxfc.platform.order.biz.*;
import com.xxfc.platform.order.contant.enumerate.OrderStatusEnum;
import com.xxfc.platform.order.contant.enumerate.OrderTypeEnum;
import com.xxfc.platform.order.entity.BaseOrder;
import com.xxfc.platform.order.pojo.order.ActivityInfoBO;
import com.xxfc.platform.order.pojo.price.OrderPriceVO;
import com.xxfc.platform.universal.constant.DictionaryKey;
import com.xxfc.platform.universal.feign.ThirdFeign;
import com.xxfc.platform.vehicle.feign.VehicleFeign;
import lombok.extern.log4j.Log4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.time.format.DateTimeFormatter;

import static com.xxfc.platform.universal.constant.DictionaryKey.APP_ORDER;

@Service
@Log4j
public class OrderActivityInfoService extends AbstractOrderHandle<OrderActivityInfoDetailBiz, ActivityInfoBO> implements UserRestInterface {


    @Autowired
    VehicleFeign vehicleFeign;

    @Autowired
    ThirdFeign thirdFeign;

    @Autowired
    OrderAccountBiz orderAccountBiz;


    @Autowired
    BaseOrderBiz baseOrderBiz;


    private static Integer IS_CHILD = 1;
    private static Integer LEVEL_DEFAULT = 0;
    private static Integer NUMBER_ZERO = 0;
    private static Long autoCancelTime;

    @Autowired
    OrderCostDetailBiz orderCostDetailBiz;

    @Autowired
    OrderTemplateBiz orderTemplateBiz;

    @Autowired
    protected UserAuthConfig userAuthConfig;

    @Autowired
    ActivityFeign activityFeign;

    @Autowired
    public HttpServletRequest request;

    private SimpleDateFormat simleFormatter = new SimpleDateFormat("YYYY-MM-dd HH:mm:ss");

    private  DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern("YYYYMMddHHmmss");

    @PostConstruct
    public void init() {
        this.channel = null;
        this.orderTypeEnum = OrderTypeEnum.ACTIVITY;
    }


    @Override
    public void initDetail(ActivityInfoBO bo) {
        super.initDetail(bo);
        initDictionary();
    }

    public void initDictionary() {
        dictionaryMap = thirdFeign.dictionaryGetAll4Map().getData();
        this.autoCancelTime = Long.valueOf(dictionaryMap.get(APP_ORDER + "_" + DictionaryKey.ACT_MEMBER).getDetail());
    }



    @Override
    public void handleDetail(ActivityInfoBO bo) {
        //设置订单状态为3
        bo.getOrder().setStatus(OrderStatusEnum.ORDER_UNPAY.getCode());
        ActivityInfo activityInfo = bo.getActivity();
        if (activityInfo != null ){
            //设置订单图片
            bo.getOrder().setPicture(activityInfo.getImg());
            //设置订单名称
            bo.getOrder().setName(activityInfo.getName());
            bo.setActivityId(activityInfo.getId());
            bo.setActivityInfo(JSON.toJSONString(activityInfo));
            ActivityInfoRuleVo activityInfoRuleVo = activityInfo.getActivityInfoRuleVo();
            if (activityInfoRuleVo != null ){
                bo.setRentFreeNum(activityInfoRuleVo.getDays());
                bo.setParentComisson(activityInfoRuleVo.getParentComisson());
            }
        }
        AppUserDTO appUserDTO = bo.getAppUserDTO();
        if (appUserDTO != null && appUserDTO.getParentId() != null)
            bo.setParentUserId(appUserDTO.getParentId());
        super.handleDetail(bo);
        //发送定时取消订单(30分钟)
        rabbitProduct.sendDelayMessage(bo.getOrder(), autoCancelTime);
    }

    @Override
    public void handleCalculate(ActivityInfoBO bo) {
        OrderPriceVO tpv = calculatePrice(bo);
        BeanUtil.copyProperties(tpv, bo.getOrder());
        BeanUtil.copyProperties(tpv, bo);
    }

    @Override
    public OrderPriceVO calculatePrice(ActivityInfoBO detail) {
        BigDecimal orderAmount = BigDecimal.ZERO;
        BigDecimal goodsAmount = BigDecimal.ZERO;
        BigDecimal realAmount = BigDecimal.ZERO;
        BigDecimal couponAmount = BigDecimal.ZERO;


        ActivityInfo activityInfo = detail.getActivity();
        BigDecimal activityAmount = BigDecimal.ZERO;
        if (activityInfo != null ) {
            ActivityInfoRuleVo activityInfoRuleVo = activityInfo.getActivityInfoRuleVo();
            if (activityInfoRuleVo != null ){
                activityAmount = activityInfoRuleVo.getPayPrice();
            }
        }

        //商品价格
        goodsAmount = goodsAmount.add(activityAmount);
        //订单总价格
        orderAmount = orderAmount.add(goodsAmount);
        //真实价格
        realAmount = realAmount.add(orderAmount).subtract(couponAmount);

        //生成订单明细
        OrderPriceVO opv = new OrderPriceVO();
        opv.setOrderAmount(orderAmount);
        opv.setGoodsAmount(goodsAmount);
        opv.setRealAmount(realAmount);
        opv.setCouponAmount(couponAmount);

        return opv;
    }



}
